# Continuous Deployment of Cloud Custodian policies using AWS CDK - Auto Configure EC2 instance profiles for Systems Manager

## Summary
This pattern demonstrates how AWS DevOps tools can be used to achieve continuous, at-scale deployment of Cloud Custodian policies to multi-account environment without provisioning a separate compute environment. We use the Cloud Development Kit (CDK)'s pipelines module to provision a CI/CD pipeline using AWS CodePipeline to orchestrate the build and testing of source code using AWS CodeBuild and the deployment of AWS resources using AWS CloudFormation stacks that are synthesized as a build artifact of the CDK code.

The specific use case focuses on a customer who wants to ensure all the EC2 instances have an IAM SSM policy attached to the instance profile so they can be managed from AWS Systems Manager. There are three main challenges that this solution addresses:

- All new instances created without an instance profile must be updated with a role that has the System Manger Core policy
- Instances created with an instance profile but without the AWS Systems Manager Core policy must be updated with the policy.
- Existing EC2 instances that have an instance profile but don’t have the policy: Currently there is no easy way to add the policy cross account/region. This is a potential blocker.

This solution addresses these by deploying and enforcing three Cloud Custodian policies:

- The first policy checks for existing EC2 instances that have an instance profile but don't have the the SSM IAM policy attached. After the policy runs, the IAM policy is attached.
- The second policy checks for existing EC2 instances without an instance profile and adds a default instance profile that has the SSM policy attached.
- Finally, the third policy creates Custodian lambda functions in the member accounts to monitor for instance profile and EC2 instance creation and ensure the SSM IAM policy is attached.

## Architecture
When developers push code to Git (AWS CodeCommit), an Amazon CloudWatch events rule triggers the CodePipeline automatically based on the Amazon CloudWatch events rule actions enabled.

At this stage, the pipeline fetches the latest code from AWS CodeCommit and the code is subsequently passed to the CI part of the pipeline handled by AWS CodeBuild (explained more clearly in next section), which performs all the end-to-end DevSecOps actions from policy syntax validation on the Cloud Custodian policies as well as running the policies in --dryrun mode to check which resources are identified by the policy, without running any actions on the resources. If there are no errors, the next stage alerts an admin (email set via the vars file) to review the changes and approve the deployment into the AWS member accounts.

### Target technology stack
**AWS CodePipeline with five stages**
- AWS CodeCommit as a Source
- AWS CodeBuild stage to bootstrap the CDK in the member accounts and deploy the required roles
- AWS CodeBuild stage to validate all required dependencies
- Approval stage to get email approval before deployment
- AWS CodeBuild stage to deploy the policy changes

### Target architecture
![cdk-cloudcustodian](cdk-cloudcustodian.svg)
## Automation and scale
- This deployment can be extended to all member accounts and regions in the Organization
- The Roles creation stack can be extended to deploy other IAM roles in the Member accounts.

## Tools:
### Utilities:
- NodeJS runtime - Runtime for NodeJS should be installed in the local system
- Git Client - Command line or desktop tool to checkout the required artifacts
- AWS CLI -  AWS Command Line Interface (CLI) is a unified tool to manage your AWS services.
- The CDK Framework for Python: This framework can be used to dynamically generate and deploy cloudformation templates using the python programming language. Ideally this should be implemented as part of a pipeline that is triggered automatically by a code merge following a pull request approval.
## Module Information
### **cloudcustodian_stack**
This module deploys a pipeline to run the Cloud Custodian tool.
1. [AWS CodePipeline](https://aws.amazon.com/codepipeline/) is used for continuous integration/continuous deployment (CI/CD). It consists of five stages.
   1. Source: The first stage is a source stage powered by an [AWS CodeCommit](https://aws.amazon.com/codecommit/) repository. All source code for this solution is stored in this repository including the SSM Cloud Custodian policies we will be deploying.
   2. Configure: The second stage is powered by [AWS CodeBuild](https://aws.amazon.com/codebuild/). In this stage, the member accounts are bootstrapped and the required Cloud Custodian cross account roles are deployed.
   3. Validate: The third stage is also powered by AWS CodeBuild. In this stage, the *cloudcustodian* command is used to run policy syntax validation on the Cloud Custodian policy files located in the 'policies' folder.
      Also, [c7n-org](https://cloudcustodian.io/docs/tools/c7n-org.html) (Cloud Custodian Multi Account Tool) is used in --dryrun mode to check which resources are identified by the policy, without running any actions on the resources.
   4. Approval: The fourth stage is an approval stage. This allows an admin to review the changes that will take place. An email notification is sent to the admin alerting them of the required approval.
   5. Deploy: The fifth and final stage is powered by AWS CodeBuild. In this stage, the [c7n-org](https://cloudcustodian.io/docs/tools/c7n-org.html) (Cloud Custodian Multi Account Tool) is used to deploy policies to destination AWS accounts. The mailer notification tool is also configured to send notifications to a pre-configured slack channel whenever the policy lambda functions take an action on the resource specified in the policy.
2. New policies can be added to the source repository and the pipeline will be triggered to deploy these policies.
3. An AWS CloudWatch Event rule watches for commits on the source repo which triggers the pipeline, and all the current policies found in the source repository are deployed to the member accounts.

### **member_account_roles_stack**
For this use-case we are deploying the Cloud Custodian policies in the member accounts. This requires having 2 roles in each member account so that CloudCustodian:
- SecurityCustodian-CrossAccount-Role: This allow CloudCustodian to deploy the policies in the member accounts in the form of lambda functions.
- SecurityCustodian-periodic-lambda-role: this is the execution role of the lambda functions that are deployed by Cloud Custodian.

## Prerequisites
To use this framework you must have the following installed:

`NodeJs` - https://nodejs.org/en/download/

`AWS CDK` - https://docs.aws.amazon.com/cdk/latest/guide/getting_started.html#getting_started_install

`AWS Cli` - https://docs.aws.amazon.com/cli/latest/userguide/install-cliv2.html

`Git` - https://git-scm.com/downloads

The install-prerequisites.sh file in this repo can be used to install all of these on Amazon linux (Excluding AWS CLI which already comes pre-installed). Make sure to run with root privileges.
Additional information can be found here:
https://docs.aws.amazon.com/cdk/latest/guide/getting_started.html#getting_started_prerequisites

## Initial setup
Clone this repo into your working directory.
Log in to your AWS Security Account and create a new repository called "devops-cdk-cloudcustodian".
Change into the project directory and setup the CodeCommit repo as origin, commit changes, and push them to the origin.
```
cd devops-cdk-cloudcustodian
git init --initial-branch=main
git add . git commit -m 'initial commit'
git remote add origin https://git-codecommit.us-east-1.amazonaws.com/v1/devops-cdk-cloudcustodian
git push origin main
```
### Create a virtualenv on MacOS and Linux:
```
$ python3 -m venv .env
```
### Activate your virtualenv
```
$ source .env/bin/activate
```
### Once the virtualenv is activated, you can install the required dependencies.
```
$ pip install -r requirements.txt
```
To add additional dependencies, for example other CDK libraries, just add to
your requirements.txt file and rerun the `pip install -r requirements.txt` command.
### Provide access to AWS
You need to ensure the AWS CLI has access to AWS. Ideally this access is granted using a role assigned to AWS CodeBuild or to an EC2 instance via an instance profile. For testing you can configure the AWS CLI using the aws configure command or set environment variables however this is not recommended for production. Also ensure this account is only granted least permissive access.
## Update the required account variables
Update the vars file with the required values.
* `var_region` = 'us-east-1'
    > Region where the pipeline will be deployed

* `var_codecommit_repo_name` = "cdk-cloudcustodian"
    > The name of the repo we are using

* `var_codecommit_branch_name` = "main"
    > The name of the branch

* `var_adminEmail` = 'notifyadmin@email.com'
    > The email address of the admin that will approve the changes.

* `var_slackWebHookUrl` = 'https://hooks.slack.com/services/T00000000/B00000000/XXXXXXXXXXXXXXXXXXXXXXXX'
    > Slack WebHook used to send CloudCustodian notification when changes are made

* `var_orgId` = 'o-yyyyyyyyyy'
    > AWS OrgId

* `security_account`='123456789011'
    > Account where the pipeline will be deployed.

* `member_accounts`=['111111111111','111111111112','111111111113']
    > List of member accounts you want to bootstrap the cdk and deploy the required roles to.

* `cdk_bootstrap_member_accounts` = True
    > Set to `True` if you want the pipeline to automatically CDK bootstrap the member accounts. If set to `True` this will also require the name of an existing role in the member accounts that can be assumed from the security account and has the required permissions to bootstrap the CDK.

    > **Set the role `cdk_bootstrap_role` = 'AWSControlTowerExecution' below.
    As you can see in our example, we are using: AWSControlTowerExecution.** This is a role that exists in every  AWS Member Account with the Security Account in the Trust Policy of the role.

* `cdk_bootstrap_role` = 'AWSControlTowerExecution'
    > Existing role in the member accounts that can be assumed from the security account and has the required permissions to bootstrap the CDK. (Only applicable if cdk_bootstrap_member_accounts is set to True)


## Bootstrap the CDK
You need to bootstrap the CDK before you can use it to deploy resources to AWS. When you run the bootstrap command, it deploys a cloudformation stack called `CDKToolkit` that creates an S3 bucket where the CFTs will be uploaded during deployment. The command is idempotent so if you are deploying as part of a pipeline you can safely run this command more than once. If an environment has already been bootstrapped, its bootstrap stack will be upgraded if necessary; otherwise, nothing happens.

This solution is using the CDK Assume Role Credential Plugin to enable this functionality. More info here: https://github.com/aws-samples/cdk-assume-role-credential-plugin

### 1.  Bootstrap the Security Account:
> Bootstrap the Security account with the cloudcustodian_stack app:
```
$ cdk bootstrap --cloudformation-execution-policies arn:aws:iam::aws:policy/AdministratorAccess
```
### 2a. Bootstrap AWS Member Accounts Automatically:
If `cdk_bootstrap_member_accounts` variable is set to `True` in the vars.py file, the AWS accounts specifed in the `member_accounts` variable will be bootstrapped automatically by the pipeline during the Configure stage. If this is desired, be sure to update the role name *cdk_bootstrap_role* with a role that you can assume from the security account and has the required permissions to bootstrap the CDK. New accounts added to the `member_accounts` variable will be automatically bootstrapped by the pipeline so the required roles can be deployed.

### 2b Bootstrap AWS Member Accounts Manually:
Alternatively, you can set the value of `cdk_bootstrap_member_accounts` to `False` and perform this step manually using the command below. Be sure to update the {security_account_id} and {role_name} values below with the name of a role that you can assume from the security account and has the required permissions to bootstrap the CDK.
```
$ cdk bootstrap -a 'python3 cloudcustodian/member_account_roles_stack.py' \
--trust {security_account_id} \
--context assume-role-credentials:writeIamRoleName={role_name} \
--context assume-role-credentials:readIamRoleName={role_name} \
--mode=ForWriting \
--context bootstrap=true \
--cloudformation-execution-policies arn:aws:iam::aws:policy/AdministratorAccess

```

You can also use other methods to bootstrap the member accounts such as using a CFT. More information on this can be found here: https://docs.aws.amazon.com/cdk/latest/guide/bootstrapping.html

## Deploy the CloudCustodian Pipeline

Once the CDK has been bootstrapped and the code has been uploded to the CodeCommit repo in the Security account, you can deploy the CloudCustodian pipeline with the following command from the root of the repo:

```
cdk deploy

```

### AWS Account Setup:

- **Security Account**: The cloudcustodian_stack is deployed here which creates the pipeline in this account. With AWS CodeCommit, the repo must be created in this account.

- **Member accounts**: The member_account_roles_stack is deployed here which create the two roles that Cloud Custodian requires to run the policies.


### File Details
 * `accounts.yml`                           In order to run c7n-org against multiple accounts, a config file must first be created containing pertinent information about the accounts.
 * `/policies/`                             Location for the Cloud Custodian policies that will be deployed when the pipeline is run.
 * `/iam_policies/*`                        Location for the IAM policy files.
 * `vars.py`                                Contains all the variables used in the process of generating the cloudformation templates
 * `cloudcustodian/cloudcustodian_stack.py`            CDK file that defines the Pipeline and other resources required to run Cloud Custodian.
 * `cloudcustodian/member_account_roles_stack.py`      CDK file that defines the required roles that need to be created in the member accounts
 * `resources/validate.sh`                  Script used in the Validate the Cloud Custodian policies and run the policies in dry-run mode to view the changes that be implemented in the Release stage.
 * `resources/deploy.sh`                    Script used in the Release stage of the pipeline to process the Cloud Custodian policies and implement the changes
 * `resources/slack_iam.j2`                 Template for the slack notification used by the Cloud Custodian mailer function.

### Deployment Commands
 Emits synthesized Cft for the cloudcustodian_stack.
 * `cdk synth -a 'python3 cloudcustodian/cloudcustodian_stack.py'`

 Deploy the resources specified in the cloudcustodian_stack
 * `cdk deploy -a 'python3 cloudcustodian/cloudcustodian_stack.py'`

Delete the resources specified in the cloudcustodian_stack (**use with caution!**)
* `cdk destroy --app 'python3 cloudcustodian/cloudcustodian_stack.py'`

List all stacks generated for member_account_roles_stack
 * `cdk ls -a 'python3 cloudcustodian/member_account_roles_stack.py'`

 List all stacks generated for IAM principals
 * `cdk ls -a 'python3 'python3 cloudcustodian/member_account_roles_stack.py'`

 Emit synthesized IamPrincipals Cft for a specific account
 * `cdk synth '*{AwsAccountID}' -a 'python3 cloudcustodian/member_account_roles_stack.py'`

 Deploy only IAM roles for a specific account
 * `cdk deploy '*{AwsAccountID}' -a 'python3 cloudcustodian/member_account_roles_stack.py' --require-approval never`

 Deploy Roles to all member accounts in the Organization
 * `cdk deploy --all -a 'python3 cloudcustodian/member_account_roles_stack.py' --require-approval never`

 Delete the Roles specified in the stack for a specific account (*use with caution!*)
 * `cdk destroy -f '*{AwsAccountID}' --app 'python3 cloudcustodian/member_account_roles_stack.py'`

 Delete the Roles specified in the stack for all member accounts in the Organization (*use with caution!*)
 * `cdk destroy -f --all --app 'python3 cloudcustodian/member_account_roles_stack.py'`

Compares deployed stacks with current state
 * `cdk diff`

 Open CDK help
 * `cdk help`

 Open CDK documentation
 * `cdk docs`

### Important Tips
- Make sure to update the variables in the `vars.py` file with the correct values
- All cdk commands should be run from the root dir of the repo