#!/.env/bin/python3
import os, sys, json
from aws_cdk import (core, aws_codebuild as codebuild,
                     aws_codecommit as codecommit,
                     aws_codepipeline as codepipeline,
                     aws_codepipeline_actions as codepipeline_actions,
                     aws_iam as iam,
                     aws_sqs as sqs)

sys.path.append(os.path.abspath(os.curdir))
from vars import *

class PipelineStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, *,
                 repo_name: str=None,  branch_name: str=None, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        adminEmail = var_adminEmail
        Region = var_region
        SlackWebHookUrl = var_slackWebHookUrl

        mailerLambdaRole = iam.Role(self, "mailerLambdaRole",
          assumed_by=iam.ServicePrincipal("lambda.amazonaws.com"),
          role_name="SecurityCustodian-mailer-role",
          managed_policies=[
                        iam.ManagedPolicy.from_aws_managed_policy_name(
                            'service-role/AWSLambdaBasicExecutionRole')
                        ])

        queue = sqs.Queue(self, "SQSQueue", queue_name="SecurityCustodian-queue",)
        queue.grant_consume_messages(mailerLambdaRole)

        orgPrincipal = iam.OrganizationPrincipal(var_orgId)
        lambdaService = iam.ServicePrincipal('lambda.amazonaws.com')

        sqsPolicyDocument = iam.PolicyStatement(
        #   "Effect" : "Allow",
          principals = [orgPrincipal],
          actions= [
            "sqs:SendMessage"
          ],
          resources = [queue.queue_arn],
        )
        queue.add_to_resource_policy(sqsPolicyDocument)

        cloudCustodianRole = iam.Role(self, "Role", role_name='SecurityCustodian-role',
          managed_policies=None,
          assumed_by=iam.ServicePrincipal("codebuild.amazonaws.com")) # required

        with open('iam_policies/cloudcustodian.json') as policy:
          PolicyDocument = json.load(policy)

        cloudCustodianPolicyDocument = iam.PolicyDocument.from_json(PolicyDocument)

        policy = iam.Policy(self, "CustodianPolicy", document=cloudCustodianPolicyDocument)
        policy.attach_to_role(cloudCustodianRole)

        queue.grant_send_messages(cloudCustodianRole)

        code = codecommit.Repository.from_repository_name(self, "ImportedRepo",
                  repo_name)

        configure_build = codebuild.PipelineProject(self, 'Configure', role=cloudCustodianRole,
            environment=codebuild.LinuxBuildImage.STANDARD_2_0,
            build_spec=codebuild.BuildSpec.from_source_filename('./codebuild/buildspec.yml')
        )

        validation_build = codebuild.PipelineProject(self, 'Validate', role=cloudCustodianRole,
            environment=codebuild.LinuxBuildImage.STANDARD_2_0,
            build_spec=codebuild.BuildSpec.from_object(dict(
                version="0.2",
                phases=dict(
                    install=dict(
                        commands=[
                            # "pip3 install c7n c7n-org"
                            ]),
                    build=dict(
                        commands=[
                            "./resources/validate.sh"
                            ])),

                    artifacts={
                        "base-directory": "output",
                        "files": [
                            "**/*"]}
             )),
            environment_variables={
                'MAILER_REGION': codebuild.BuildEnvironmentVariable(value=Region),
                # 'BUILD_ARTIFACT_BUCKET': codebuild.BuildEnvironmentVariable(value=artifact_bucket.bucket_name)
            }
        )

        cloudcustodian_build = codebuild.PipelineProject(self, 'CloudCustodianBuild', role=cloudCustodianRole,
            environment=codebuild.LinuxBuildImage.STANDARD_2_0,
            build_spec=codebuild.BuildSpec.from_object(dict(
                version="0.2",
                phases=dict(
                    install=dict(
                        commands=[
                            ]),
                    build=dict(
                        commands=[
                            "./resources/deploy.sh"
                            ])),
                    artifacts={
                        "base-directory": "output",
                        "files": [
                            "**/*"]}
                )),
            environment_variables={
                "MAILER_REGION": codebuild.BuildEnvironmentVariable(value=Region),
                "MAILER_ROLE_ARN": codebuild.BuildEnvironmentVariable(value=mailerLambdaRole.role_arn),
                "TRANSPORT_QUEUE_URL": codebuild.BuildEnvironmentVariable(value=queue.queue_url),
                "SLACK_WEBHOOK_URL": codebuild.BuildEnvironmentVariable(value=SlackWebHookUrl)
            }
        )

        # cloudcustodian_build.role.arn
        cloudcustodian_build.add_to_role_policy(iam.PolicyStatement(
            effect=iam.Effect.ALLOW,
            resources=["arn:aws:iam::*:role/SecurityCustodian-CrossAccount-Role",
            "arn:aws:iam::*:role/cdk-*-role*"],
            actions=["sts:AssumeRole"]
        ))

        if cdk_bootstrap_member_accounts:
            cloudcustodian_build.add_to_role_policy(iam.PolicyStatement(
                effect=iam.Effect.ALLOW,
                resources=["arn:aws:iam::*:role/"+cdk_bootstrap_role],
                actions=["sts:AssumeRole"]
            ))

        source_output = codepipeline.Artifact()
        CloudCustodian_configure_output = codepipeline.Artifact("CloudCustodianConfigureOutput")
        CloudCustodian_build_output = codepipeline.Artifact("CloudCustodianBuildOutput")
        C7nMailer_build_output = codepipeline.Artifact("C7nMailerBuildOutput")

        codepipeline.Pipeline(self, "Pipeline",
            stages=[
                codepipeline.StageProps(stage_name="Source",
                    actions=[
                        codepipeline_actions.CodeCommitSourceAction(
                            action_name="CodeCommit_Source",
                            repository=code,
                            branch=branch_name,
                            output=source_output)]),
                codepipeline.StageProps(stage_name="Configure",
                    actions=[
                        codepipeline_actions.CodeBuildAction(
                            action_name="ConfigureMemberAccounts",
                            project=configure_build,
                            input=source_output,
                            outputs=[CloudCustodian_configure_output])]),
                codepipeline.StageProps(stage_name="Validate",
                    actions=[
                        codepipeline_actions.CodeBuildAction(
                            action_name="PolicyValidation",
                            project=validation_build,
                            input=source_output,
                            outputs=[C7nMailer_build_output])]),
                codepipeline.StageProps(stage_name="Approval",
                    actions=[
                        codepipeline_actions.ManualApprovalAction(
                            action_name="Approval",
                            notify_emails=[adminEmail]
                            )]),
                codepipeline.StageProps(stage_name="Release",
                    actions=[
                        codepipeline_actions.CodeBuildAction(
                            action_name="CloudCustodian_Build",
                            project=cloudcustodian_build,
                            input=source_output,
                            outputs=[CloudCustodian_build_output])]),
            ]
        )

env_USA = core.Environment(account=security_account, region=var_region)

app = core.App()
core.Tags.of(app).add("solution", "cdk-cloudcustodian")
PipelineStack(app, "cloudcustodian",
    repo_name=var_codecommit_repo_name,
    branch_name=var_codecommit_branch_name,
    env=env_USA)
app.synth()